import datetime as dt
import numpy as np
import os

from OMIEData.FileReaders.marginal_price_file_reader import MarginalPriceFileReader
from OMIEData.DataImport.omie_data_importer_from_folder import (
    OMIEDataImporterFromFolder,
)
from OMIEData.Enums.all_enums import DataTypeInMarginalPriceFile
import pandas as pd


def is_equal_float(x1: float, x2: float, tolerance=1e-4):
    return np.abs(x1 - x2) < tolerance


def test_all_keys_in_df():
    folder = os.path.join(os.path.dirname(__file__), "InputTesting")
    filename = os.path.join(folder, "PMD_20060101.txt")

    reader = MarginalPriceFileReader()
    keys = reader.get_keys()
    df = reader.get_data_from_file(filename=filename)

    # data is list of dictionaries
    columns = df.columns.to_list()
    for k in keys:
        assert k in columns, "Key: " + k + " not found."


def test_check_correct_values():
    folder = os.path.join(os.path.dirname(__file__), "InputTesting")
    filename = os.path.join(folder, "PMD_20060101.txt")

    # File contains
    #'Precio marginal (Cent/kWh);  6,694;  4,888;  4,525;  4,371;  3,870;  3,777;  3,611;  1,000;  0,500;  1,000;  1,000;\
    #  1,954;  3,755;  3,777;  3,777;  3,755;  3,755;  3,755;  4,788;  5,600;  6,725;  7,001;  6,637;  7,617;
    #' Energía en el programa resultante de la casación (MWh);  26.377;  26.070;  24.916;  23.761;  22.814;  22.116;  \
    # 21.415;  20.712;  19.438;  19.699;  20.583;  21.119;  21.741;  22.264;  22.359;  21.763;  21.463;  21.610;  23.872;\
    #  24.322;  24.993;  25.064;  24.792;  25.373;'

    data = MarginalPriceFileReader().get_data_from_file(filename=filename)

    assert is_equal_float(
        data.iloc[0]["H7"], 36.11, tolerance=1e-3
    ), "Data is corrupted"
    assert is_equal_float(
        data.iloc[1]["H9"], 19438, tolerance=1e-3
    ), "Data is corrupted"


def test_dump_to_dataframe(verbose=False):
    folder = os.path.join(os.path.dirname(__file__), "InputTesting")
    file_reader = MarginalPriceFileReader()
    dumper = OMIEDataImporterFromFolder(absolute_path=folder, file_reader=file_reader)
    df = dumper.read_to_dataframe()

    # show dataframe
    if verbose:
        print(df)

    # 2006-1-1 file
    value = float(
        df.loc[
            (df.DATE == dt.date(2006, 1, 1))
            & (df.CONCEPT == str(DataTypeInMarginalPriceFile.PRICE_SPAIN)),
            "H1",
        ]
    )
    assert is_equal_float(value, 66.94, tolerance=1e-6), "Data is corrupt"

    value = float(
        df.loc[
            (df.DATE == dt.date(2006, 1, 1))
            & (df.CONCEPT == str(DataTypeInMarginalPriceFile.ENERGY_IBERIAN)),
            "H23",
        ]
    )
    assert is_equal_float(value, 24792, tolerance=1e-6), "Data is corrupt"

    # 2009-6-1 file
    value = float(
        df.loc[
            (df.DATE == dt.date(2009, 6, 1))
            & (df.CONCEPT == str(DataTypeInMarginalPriceFile.PRICE_PORTUGAL)),
            "H2",
        ]
    )
    assert is_equal_float(value, 37.60, tolerance=1e-6), "Data is corrupt"

    slice = df.loc[
        (df.DATE == dt.date(2009, 6, 1))
        & (df.CONCEPT == str(DataTypeInMarginalPriceFile.ENERGY_IBERIAN)),
        :,
    ]
    energies = {
        "H1": 28325.8,
        "H2": 26201.2,
        "H3": 24651.6,
        "H4": 23788.6,
        "H5": 23565.4,
        "H6": 24149.6,
        "H7": 26386.0,
        "H8": 29016.5,
        "H9": 33149.4,
        "H10": 36289.9,
        "H11": 38527.1,
        "H12": 39362.9,
        "H13": 40102.1,
        "H14": 39556.3,
        "H15": 37944.9,
        "H16": 37503.3,
        "H17": 37355.8,
        "H18": 37310.8,
        "H19": 36740.7,
        "H20": 36079.6,
        "H21": 36029.6,
        "H22": 37617.5,
        "H23": 37147.9,
        "H24": 33897.1,
    }
    for k, v in energies.items():
        assert is_equal_float(v, float(slice.get(k)), tolerance=1e-6), "Data is corrupt"

    # 2020-10-22 file
    slice = df.loc[
        (df.DATE == dt.date(2020, 10, 22))
        & (
            df.CONCEPT
            == str(DataTypeInMarginalPriceFile.ENERGY_IBERIAN_WITH_BILLATERAL)
        ),
        :,
    ]
    energies = [
        29435.7,
        27853.8,
        27293.8,
        26948.5,
        26871.7,
        27047.6,
        28605.8,
        31899.4,
        33804.2,
        35118.1,
        35916.7,
        36518.3,
        37036.5,
        37006.9,
        36251.2,
        35312.6,
        34689.7,
        34358.7,
        33956.2,
        35138.1,
        37878.5,
        36955.3,
        33874.5,
        30856.7,
    ]
    for i, v in enumerate(energies):
        assert is_equal_float(
            energies[i], float(slice.get("H" + f"{i + 1:01}")), tolerance=1e-6
        ), "Data is corrupt"

    slice = df.loc[
        (df.DATE == dt.date(2020, 10, 22))
        & (df.CONCEPT == str(DataTypeInMarginalPriceFile.PRICE_SPAIN)),
        :,
    ]
    prices = [
        39.55,
        35.00,
        33.07,
        32.68,
        32.68,
        33.08,
        40.11,
        47.13,
        49.53,
        52.49,
        52.43,
        50.44,
        49.08,
        47.50,
        46.58,
        45.95,
        46.11,
        48.49,
        50.68,
        56.63,
        53.56,
        49.04,
        47.20,
        46.30,
    ]
    for i, v in enumerate(prices):
        assert is_equal_float(
            prices[i], float(slice.get("H" + f"{i + 1:01}")), tolerance=1e-6
        ), "Data is corrupt"


def test_day_with_23_hours():
    folder = os.path.join(os.path.dirname(__file__), "InputTesting")
    filename = os.path.join(folder, "PrecioMD_OMIE_20200329.txt")

    # File contains
    #';1;2;3;4;5;6;7;8;9;10;11;12;13;14;15;16;17;18;19;20;21;22;23;
    # Precio marginal (Cent/kWh);  2,002;  1,762;  1,562;  1,527;  1,482;  1,475;  1,762;  2,149;  4,167;  4,340;  \
    # 4,276;  4,276;  3,500;  3,585;  3,200;  3,300;  3,400;  2,192;  3,300;  5,000;  4,426;  4,197;  3,673;  4,340;
    # Demanda+bombeos (MWh);  23.669;  21.263;  19.804;  19.187;  18.774;  18.666;  20.321;  22.565;  24.720;  25.838;  \
    # 26.405;  26.603;  26.194;  25.455;  23.956;  24.020;  24.041;  24.599;  26.224;  28.132;  27.617;  26.571;  \
    # 24.909;  24.664;
    # ;;;;;;;;;;;;;;;;;;;;;;;;;'

    df = MarginalPriceFileReader().get_data_from_file(filename=filename)
    assert pd.isna(df.iloc[0]["H24"]), "H24 should be NaN."


def test_day_with_25_hours():
    # TODO: This test can fail in some architecture  because of the encoding of the input file.
    #  Let´s see if more users/developers have problems with it.
    folder = os.path.join(os.path.dirname(__file__), "InputTesting")
    filename = os.path.join(folder, "PrecioMD_OMIE_20223010.txt")

    # File contains
    #';1;2;3;4;5;6;7;8;9;10;11;12;13;14;15;16;17;18;19;20;21;22;23;24;25;
    # ;;;;;;;;;;;;;;;;;;;;;;;;;'

    df = MarginalPriceFileReader().get_data_from_file(filename=filename)
    assert not pd.isna(df.iloc[0]["H25"]), "H24 should not be NaN."


def test_20030802():
    folder = os.path.join(os.path.dirname(__file__), "InputTesting")
    filename = os.path.join(folder, "PMD_20030802.txt")

    # File contains
    # ;1;2;3;4;5;6;7;8;9;10;11;12;13;14;15;16;17;18;19;20;21;22;23;24;
    # Precio marginal (Cent/kWh);  4,553;  3,700;  2,955;  2,607;  2,406;  2,206;  2,089;  2,089;  2,647;  3,956;  \
    # 4,662;  4,934;  5,096;  5,149;  5,000;  4,934;  4,900;  4,908;  4,908;  4,908;  4,800;  5,149;  5,309;  4,934;
    # Demanda+bombeos (MWh);  24.623;  22.721;  22.019;  21.657;  21.307;  20.919;  20.989;  20.889;  21.350;  22.025;\
    # 24.378;  25.414;  25.873;  25.947;  25.103;  24.215;  23.711;  23.506;  23.437;  23.406;  23.543;  24.854;  \
    # 25.507;  24.181;
    # ;;;;;;;;;;;;;;;;;;;;;;;;
    input = MarginalPriceFileReader().get_data_from_file(filename=filename)
    concepts = list(input["CONCEPT"])

    assert (
        str(DataTypeInMarginalPriceFile.ENERGY_IBERIAN) in concepts
    ), "{} has to be one of the concepts read".format(
        DataTypeInMarginalPriceFile.ENERGY_IBERIAN
    )
    assert (
        str(DataTypeInMarginalPriceFile.PRICE_SPAIN) in concepts
    ), "{} has to be one of the concepts read".format(
        DataTypeInMarginalPriceFile.PRICE_SPAIN
    )


def test_20040101():
    folder = os.path.join(os.path.dirname(__file__), "InputTesting")
    filename = os.path.join(folder, "PMD_20040101.txt")

    # File contains
    # OMEL - Mercado de electricidad;Fecha Emisión :31/12/2003 - 10:23;;01/01/2004;Precio del mercado diario (cent/kWh)\
    # ;;;;
    #
    # ;1;2;3;4;5;6;7;8;9;10;11;12;13;14;15;16;17;18;19;20;21;22;23;24;
    # Precio marginal (Cent/kWh);  2,899;  2,823;  2,548;  2,300;  1,654;  1,468;  1,454;  1,167;  0,757;  0,287;  \
    # 1,001;  0,937;  1,127;  1,217;  1,197;  1,012;  0,917;  1,022;  1,468;  2,101;  2,101;  2,300;  2,101;  2,300;
    # Demanda+bombeos (MWh);  21.704;  20.248;  18.415;  16.699;  15.449;  14.677;  14.539;  14.390;  14.596;  14.739;\
    # 14.908;  16.360;  16.826;  16.855;  17.001;  16.914;  16.666;  17.398;  18.787;  20.146;  20.512;  20.935;\
    # 20.381;  20.493;
    # ;;;;;;;;;;;;;;;;;;;;;;;;;
    #

    df = MarginalPriceFileReader().get_data_from_file(filename=filename)

    prices = [
        10 * x
        for x in [
            2.899,
            2.823,
            2.548,
            2.300,
            1.654,
            1.468,
            1.454,
            1.167,
            0.757,
            0.287,
            1.001,
            0.937,
            1.127,
            1.217,
            1.197,
            1.012,
            0.917,
            1.022,
            1.468,
            2.101,
            2.101,
            2.300,
            2.101,
            2.300,
        ]
    ]

    for row in df.itertuples():
        if row.CONCEPT == str(DataTypeInMarginalPriceFile.PRICE_SPAIN):
            for i, v in enumerate(prices):
                assert is_equal_float(prices[i], float(row[i + 3]), tolerance=1e-6), (
                    "Data is corrupt: " + row.CONCEPT + " (H" + f"{i + 1:01})"
                )


def test_20040101_from_response():
    # File contains
    # OMEL - Mercado de electricidad;Fecha Emisión :31/12/2003 - 10:23;;01/01/2004;Precio del mercado diario (cent/kWh)\
    # ;;;;
    #
    # ;1;2;3;4;5;6;7;8;9;10;11;12;13;14;15;16;17;18;19;20;21;22;23;24;
    # Precio marginal (Cent/kWh);  2,899;  2,823;  2,548;  2,300;  1,654;  1,468;  1,454;  1,167;  0,757;  0,287;  \
    # 1,001;  0,937;  1,127;  1,217;  1,197;  1,012;  0,917;  1,022;  1,468;  2,101;  2,101;  2,300;  2,101;  2,300;
    # Demanda+bombeos (MWh);  21.704;  20.248;  18.415;  16.699;  15.449;  14.677;  14.539;  14.390;  14.596;  14.739;\
    # 14.908;  16.360;  16.826;  16.855;  17.001;  16.914;  16.666;  17.398;  18.787;  20.146;  20.512;  20.935;\
    # 20.381;  20.493;
    # ;;;;;;;;;;;;;;;;;;;;;;;;;
    #

    prices = [
        10 * x
        for x in [
            2.899,
            2.823,
            2.548,
            2.300,
            1.654,
            1.468,
            1.454,
            1.167,
            0.757,
            0.287,
            1.001,
            0.937,
            1.127,
            1.217,
            1.197,
            1.012,
            0.917,
            1.022,
            1.468,
            2.101,
            2.101,
            2.300,
            2.101,
            2.300,
        ]
    ]

    from OMIEData.Downloaders.marginal_price_downloader import MarginalPriceDownloader

    responses = list(
        MarginalPriceDownloader().url_responses(
            date_ini=dt.date(2004, 1, 1), date_end=dt.date(2004, 1, 1)
        )
    )

    df = MarginalPriceFileReader().get_data_from_response(response=responses[0])

    for row in df.itertuples():
        if row.CONCEPT == str(DataTypeInMarginalPriceFile.PRICE_SPAIN):
            for i, v in enumerate(prices):
                assert is_equal_float(prices[i], float(row[i + 3]), tolerance=1e-6), (
                    "Data is corrupt: " + row.CONCEPT + " (H" + f"{i + 1:01})"
                )


def run_all_tests():
    import warnings

    test_functions = [
        test_all_keys_in_df,
        test_check_correct_values,
        test_dump_to_dataframe,
        test_day_with_23_hours,
        test_20030802,
        test_20040101,
    ]
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", category=FutureWarning)
        warnings.filterwarnings("ignore", category=DeprecationWarning)
        for test_func in [test_functions]:
            try:
                test_func()
            except (AssertionError, UnicodeDecodeError, TypeError, IndexError) as e:
                print(f"{test_func.__name__} failed: {e}")
                raise e


if __name__ == "__main__":
    run_all_tests()



import datetime as dt


class OMIEDownloader:

    def get_complete_url(self) -> str:
        pass

    def download_data(self, date_ini: dt.date, date_end: dt.date, output_folder: str, verbose=False) -> int:
        pass

    def url_responses(self, date_ini: dt.date, date_end: dt.date, verbose=False):
        pass



from enum import Enum, auto


class SystemType(Enum):

    SPAIN = 1
    PORTUGAL = 2
    IBERIAN = 9


class TechnologyType(Enum):

    COAL = auto()
    FUEL_GAS = auto()
    SELF_PRODUCER = auto()
    NUCLEAR = auto()
    HYDRO = auto()
    COMBINED_CYCLE = auto()
    WIND = auto()
    THERMAL_SOLAR = auto()
    PHOTOVOLTAIC_SOLAR = auto()
    RESIDUALS = auto()
    IMPORT = auto()
    IMPORT_WITHOUT_MIBEL = auto()

    __dict_concept_str__ = {COAL: ('COAL', 'CARBÓN'),
                            FUEL_GAS: ('FUEL_GAS', 'FUEL-GAS'),
                            SELF_PRODUCER: ('SELF_PRODUCER', 'AUTOPRODUCTOR'),
                            NUCLEAR: ('NUCLEAR', 'NUCLEAR'),
                            HYDRO: ('HYDRO', 'HIDRÁULICA'),
                            COMBINED_CYCLE: ('COMBINED_CYCLE', 'CICLO COMBINADO'),
                            WIND: ('WIND', 'EÓLICA'),
                            THERMAL_SOLAR: ('THERMAL_SOLAR', 'SOLAR TÉRMICA'),
                            PHOTOVOLTAIC_SOLAR: ('PHOTOVOLTAIC_SOLAR', 'SOLAR FOTOVOLTAICA'),
                            RESIDUALS: ('RESIDUALS', 'COGENERACIÓN/RESIDUOS/MINI HIDRA'),
                            IMPORT: ('IMPORT', 'IMPORTACIÓN INTER.'),
                            IMPORT_WITHOUT_MIBEL: ('IMPORT_WITHOUT_MIBEL', 'IMPORTACIÓN INTER. SIN MIBEL')}

    def __str__(self):
        return self.__dict_concept_str__[self.value][0]

    def name_in_file(self):
        return self.__dict_concept_str__[self.value][1]


class DataTypeInMarginalPriceFile(Enum):

    PRICE_SPAIN = auto()
    PRICE_PORTUGAL = auto()
    ENERGY_IBERIAN = auto()
    ENERGY_IBERIAN_WITH_BILLATERAL = auto()

    __dict_concept_str__ = {PRICE_SPAIN: 'PRICE_SP',
                            PRICE_PORTUGAL: 'PRICE_PT',
                            ENERGY_IBERIAN: 'ENER_IB',
                            ENERGY_IBERIAN_WITH_BILLATERAL: 'ENER_IB_BILLAT'}

    def __str__(self):
        return self.__dict_concept_str__[self.value]


import datetime as dt
from babel.numbers import parse_decimal
import re
import numpy as np
import pandas as pd
from OMIEData.Enums.all_enums import DataTypeInMarginalPriceFile
from OMIEData.FileReaders.omie_file_reader import OMIEFileReader
from requests import Response


class AdjustmentPriceFileReader(OMIEFileReader):
    __dic_static_concepts__ = {
        "Precio de ajuste en el sistema portugués (EUR/MWh)": [
            DataTypeInMarginalPriceFile.PRICE_PORTUGAL,
            1.0,
        ]
    }

    __key_list_retrieve__ = [
        "DATE",
        "CONCEPT",
        "H1",
        "H2",
        "H3",
        "H4",
        "H5",
        "H6",
        "H7",
        "H8",
        "H9",
        "H10",
        "H11",
        "H12",
        "H13",
        "H14",
        "H15",
        "H16",
        "H17",
        "H18",
        "H19",
        "H20",
        "H21",
        "H22",
        "H23",
        "H24",
        "H25",
    ]

    __dateFormatInFile__ = "%d/%m/%Y"
    __localeInFile__ = "en_DK.UTF-8"

    def __init__(self, types=None):
        self.conceptsToLoad = (
            [v for v in DataTypeInMarginalPriceFile] if not types else types
        )

    def get_keys(self):
        return AdjustmentPriceFileReader.__key_list_retrieve__

    def get_data_from_response(self, response: Response) -> pd.DataFrame:
        res = pd.DataFrame(columns=self.get_keys())

        # from first line we get the units and the price date. We just look at the date
        lines = response.text.split("\n")
        matches = re.findall("\d\d/\d\d/\d\d\d\d", lines.pop(0))  # noqa: W605
        if not (len(matches) == 2):
            pass
        else:
            # The second date is the one we want
            date = dt.datetime.strptime(
                matches[1], AdjustmentPriceFileReader.__dateFormatInFile__
            ).date()

            # Process all the lines

            while lines:
                # read following line
                line = lines.pop(0)
                splits = line.split(sep=";")
                first_col = splits[0]

                if (
                    first_col
                    in AdjustmentPriceFileReader.__dic_static_concepts__.keys()
                ):
                    concept_type = AdjustmentPriceFileReader.__dic_static_concepts__[
                        first_col
                    ][0]

                    if concept_type in self.conceptsToLoad:
                        units = AdjustmentPriceFileReader.__dic_static_concepts__[
                            first_col
                        ][1]

                        dico = self._process_line(
                            date=date,
                            concept=concept_type,
                            values=splits[1:],
                            multiplier=units,
                        )
                        res = pd.concat([res, pd.DataFrame([dico])], ignore_index=True)

            return res

    def get_data_from_file(self, filename: str) -> pd.DataFrame:
        # Method yield each dictionary one by one
        res = pd.DataFrame(columns=self.get_keys())
        file = open(filename, "r")

        # from first line we get the units and the price date. We just look at the date
        line = file.readline()
        matches = re.findall("\d\d/\d\d/\d\d\d\d", line)  # noqa: W605
        if not (len(matches) == 2):
            pass
        else:
            # The second date is the one we want
            date = dt.datetime.strptime(
                matches[1], AdjustmentPriceFileReader.__dateFormatInFile__
            ).date()

            # Process all the lines
            while line:
                # read following line
                line = file.readline()
                splits = line.split(sep=";")
                first_col = splits[0]

                if (
                    first_col
                    in AdjustmentPriceFileReader.__dic_static_concepts__.keys()
                ):
                    concept_type = AdjustmentPriceFileReader.__dic_static_concepts__[
                        first_col
                    ][0]

                    if concept_type in self.conceptsToLoad:
                        units = AdjustmentPriceFileReader.__dic_static_concepts__[
                            first_col
                        ][1]

                        dico = self._process_line(
                            date=date,
                            concept=concept_type,
                            values=splits[1:],
                            multiplier=units,
                        )
                        res = pd.concat([res, pd.DataFrame([dico])], ignore_index=True)

            return res

    def _process_line(
        self,
        date: dt.date,
        concept: DataTypeInMarginalPriceFile,
        values: list,
        multiplier=1.0,
    ) -> dict:
        key_list = AdjustmentPriceFileReader.__key_list_retrieve__

        result = dict.fromkeys(self.get_keys())
        result[key_list[0]] = date
        result[key_list[1]] = str(concept)

        for i, v in enumerate(values, start=1):
            if i > 25:
                break  # Jump if 25-hour day or spaces ..
            try:
                f = multiplier * float(parse_decimal(v, locale=self.__localeInFile__))
            except:
                if i == 24:
                    # Day with 23-hours.
                    result[key_list[25]] = np.nan
                    result[key_list[26]] = np.nan
                elif i == 25:
                    # Day with 25-hours.
                    result[key_list[26]] = np.nan
                else:
                    raise
            else:
                result[key_list[i + 1]] = f

        return result




from OMIEData.Downloaders.general_omie_downloader import GeneralOMIEDownloader


class IntraDayPriceDownloader(GeneralOMIEDownloader):

    url_year = 'AGNO_YYYY'
    url_month = '/MES_MM/TXT/'
    url_name = 'INT_PIB_EV_H_1_SS_DD_MM_YYYY_DD_MM_YYYY.TXT'
    output_mask = 'PrecioIntra_SS_YYYYMMDD.txt'

    def __init__(self, session: int):

        str_session = f'{session:01}'
        self.output_mask = self.output_mask.replace('SS', str_session)

        url1 = self.url_year + self.url_month + self.url_name
        url1 = url1.replace('SS', str_session)

        GeneralOMIEDownloader.__init__(self,
                                       url_mask=url1,
                                       output_mask=self.output_mask)




import requests as req
import datetime as dt
import os
from OMIEData.Downloaders.omie_downloader import OMIEDownloader


class GeneralOMIEDownloader(OMIEDownloader):

    _base_url = 'https://www.omie.es/sites/default/files/dados/'
    url_mask: str
    output_folder: str
    output_mask: str

    def __init__(self, url_mask: str, output_mask: str):

        self.url_mask = url_mask
        self.output_mask = output_mask

    def get_complete_url(self) -> str:
        return self._base_url + self.url_mask

    def download_data(self, date_ini: dt.datetime, date_end: dt.datetime, output_folder: str, verbose=False) -> int:

        error = 0
        dt_aux = date_ini

        while dt_aux <= date_end:

            dd = f'{dt_aux.day:02}'
            mm = f'{dt_aux.month:02}'
            yyyy = f'{dt_aux.year:04}'

            # There could be errors when downloading or writtng to file... try-catch ??
            url_aux = self.get_complete_url()
            url_aux = url_aux.replace('DD', dd).replace('MM', mm).replace('YYYY', yyyy)

            file_aux = self.output_mask.replace('DD', dd).replace('MM', mm).replace('YYYY', yyyy)
            file_aux = os.path.join(output_folder, file_aux)

            if verbose:
                print('Downloading {} ...'.format(url_aux))
            response = req.get(url_aux, allow_redirects=True)

            if verbose:
                print('Copying to {} ...'.format(file_aux))
            f = open(file_aux, 'wb').write(response.content)

            dt_aux = dt_aux + dt.timedelta(days=+1)

        return error

    def url_responses(self, date_ini: dt.datetime, date_end: dt.datetime, verbose=False):

        dt_aux = date_ini

        while dt_aux <= date_end:

            dd = f'{dt_aux.day:02}'
            mm = f'{dt_aux.month:02}'
            yyyy = f'{dt_aux.year:04}'

            # There could be errors when downloading or writing to file... try-catch ??
            url_aux = self.get_complete_url()
            url_aux = url_aux.replace('DD', dd).replace('MM', mm).replace('YYYY', yyyy)

            if verbose:
                print('Requesting {} ...'.format(url_aux))

            yield req.get(url_aux, allow_redirects=True)

            dt_aux = dt_aux + dt.timedelta(days=+1)



import datetime as dt
import pandas as pd
# import numpy as np

from OMIEData.DataImport.omie_data_importer_from_responses import OMIEDataImporterFromResponses
from OMIEData.Downloaders.supply_demand_curve_downloader import SupplyDemandCurveDownloader
from OMIEData.FileReaders.supply_demand_curve_file_reader import SupplyDemandCurvesReader


class OMIESupplyDemandCurvesImporter(OMIEDataImporterFromResponses):

    def __init__(self,
                 date_ini: dt.date,
                 date_end: dt.date,
                 hour: int):

        super().__init__(date_ini=date_ini,
                         date_end=date_end,
                         file_downloader=SupplyDemandCurveDownloader(hour=hour),
                         file_reader=SupplyDemandCurvesReader())

    def read_to_dataframe(self, verbose=False) -> pd.DataFrame:

        df = super().read_to_dataframe(verbose=verbose)
        #TODO: we have process the data-frame to generate another in a better format.

        return df



import datetime as dt

from OMIEData.DataImport.omie_data_importer_from_responses import (
    OMIEDataImporterFromResponses,
)
from OMIEData.Downloaders.adjustment_price_downloader import AdjustmentPriceDownloader
from OMIEData.FileReaders.adjustment_price_file_reader import AdjustmentPriceFileReader


class OMIEAdjustmentPriceFileImporter(OMIEDataImporterFromResponses):
    def __init__(self, date_ini: dt.date, date_end: dt.date):
        super().__init__(
            date_ini=date_ini,
            date_end=date_end,
            file_downloader=AdjustmentPriceDownloader(),
            file_reader=AdjustmentPriceFileReader(),
        )



import pandas as pd
from requests import Response
from io import BytesIO

from OMIEData.FileReaders.omie_file_reader import OMIEFileReader
from OMIEData.Enums.all_enums import TechnologyType


class EnergyByTechnologyHourlyFileReader(OMIEFileReader):

    def __init__(self, types=None):

        self.conceptsToLoad = [v for v in TechnologyType] if not types else types

        self._dict_column_concept = {'Fecha': 'DATE',
                                     'Hora': 'HOUR',
                                     'CARBÓN': 'COAL',
                                     'FUEL-GAS': 'FUEL_GAS',
                                     'AUTOPRODUCTOR': 'SELF_PRODUCER',
                                     'NUCLEAR': 'NUCLEAR',
                                     'HIDRÁULICA': 'HYDRO',
                                     'CICLO COMBINADO': 'COMBINED_CYCLE',
                                     'EÓLICA': 'WIND',
                                     'SOLAR TÉRMICA': 'THERMAL_SOLAR',
                                     'SOLAR FOTOVOLTAICA': 'PHOTOVOLTAIC_SOLAR',
                                     'COGENERACIÓN/RESIDUOS/MINI HIDRA': 'RESIDUALS',
                                     'IMPORTACIÓN INTER.': 'IMPORT',
                                     'IMPORTACIÓN INTER. SIN MIBEL': 'IMPORT_WITHOUT_MIBEL'}

    def get_keys(self) -> list:

        key_list_retrieve = ['DATE', 'HOUR']
        key_list_retrieve.extend([str(v) for v in self.conceptsToLoad])
        return key_list_retrieve

    def get_data_from_response(self, response: Response) -> pd.DataFrame:
        return self._get_data_from_file_like(file_like=BytesIO(response.content))

    def get_data_from_file(self, filename: str) -> pd.DataFrame:
        return self._get_data_from_file_like(file_like=filename)

    def _get_data_from_file_like(self, file_like) -> pd.DataFrame:
        df = pd.read_csv(file_like, sep=';', skiprows=2, header=0, encoding='latin-1', skipfooter=1, engine='python',
                         decimal=",", thousands='.')
        df = df.rename({k: v for k, v in self._dict_column_concept.items()}, axis=1)
        df = df[[x for x in self.get_keys()]]

        return df




import os
from OMIEData.FileReaders.energy_by_technology_files_reader import EnergyByTechnologyHourlyFileReader


def test_all_keys_in_df():

    folder = os.path.join(os.path.dirname(__file__), 'InputTesting')
    filename = os.path.join(folder, 'EnergyByTechnology_9_20201113.TXT')

    reader = EnergyByTechnologyHourlyFileReader()
    keys = reader.get_keys()
    df = reader.get_data_from_file(filename=filename)

    # data is list of dictionaries
    columns = df.columns.to_list()
    for k in keys:
        assert k in columns, 'Key: ' + k + ' not found.'


def test_check_correct_values():

    folder = os.path.join(os.path.dirname(__file__), 'InputTesting')
    filename = os.path.join(folder, 'EnergyByTechnology_9_20201113.TXT')

    df = EnergyByTechnologyHourlyFileReader().get_data_from_file(filename=filename)
    assert df['NUCLEAR'][23] == 6088.9




from OMIEData.Downloaders.general_omie_downloader import GeneralOMIEDownloader


class SupplyDemandCurveDownloader(GeneralOMIEDownloader):

    url_year = 'AGNO_YYYY'
    url_month = '/MES_MM/TXT/'
    url_name = 'INT_CURVA_ACUM_UO_MIB_1_HH_DD_MM_YYYY_DD_MM_YYYY.TXT'
    output_mask = 'OfferAndDemandCurve_HH_YYYYMMDD.txt'

    def __init__(self, hour: int):

        str_hour = f'{hour:01}'
        self.output_mask = self.output_mask.replace('HH', str_hour)

        url1 = self.url_year + self.url_month + self.url_name
        url1 = url1.replace('HH', str_hour)

        GeneralOMIEDownloader.__init__(self, url_mask=url1, output_mask=self.output_mask)



import pandas as pd
from requests import Response
from io import BytesIO

from OMIEData.FileReaders.omie_file_reader import OMIEFileReader


class SupplyDemandCurvesReader(OMIEFileReader):

    def __init__(self):

        self._dict_column_concept = {'Fecha': 'DATE',
                                     'Hora': 'HOUR',
                                     'Pais': 'COUNTRY',
                                     'Unidad': 'UNIT',
                                     'Tipo Oferta': 'OFFER_TYPE',
                                     'Energía Compra/Venta': 'ENERGY',
                                     'Precio Compra/Venta': 'PRICE',
                                     'Ofertada (O)/Casada (C)': 'MATCHED'}

    def get_keys(self) -> list:
        return list(self._dict_column_concept.values())

    def get_data_from_response(self, response: Response) -> pd.DataFrame:
        return self._get_data_from_file_like(file_like=BytesIO(response.content))

    def get_data_from_file(self, filename: str) -> pd.DataFrame:
        return self._get_data_from_file_like(file_like=filename)

    def _get_data_from_file_like(self, file_like) -> pd.DataFrame:
        df = pd.read_csv(file_like, sep=';', skiprows=2, header=0, encoding='latin-1', skipfooter=1, engine='python',
                         decimal=",", thousands='.')
        df = df.rename({k: v for k, v in self._dict_column_concept.items()}, axis=1)
        df = df[[x for x in self.get_keys()]]

        return df



import datetime as dt
import re
from babel.numbers import parse_decimal
import pandas as pd
import numpy as np

from requests import Response
from OMIEData.Enums.all_enums import DataTypeInMarginalPriceFile
from OMIEData.FileReaders.omie_file_reader import OMIEFileReader


class MarginalPriceFileReader(OMIEFileReader):

    # Static or class variables
    __dic_static_concepts__ = {
        'Precio marginal (Cent/kWh)':
            [DataTypeInMarginalPriceFile.PRICE_SPAIN, 10.0],
        'Precio marginal (EUR/MWh)':
            [DataTypeInMarginalPriceFile.PRICE_SPAIN, 1.0],
        'Precio marginal en el sistema español (Cent/kWh)':
            [DataTypeInMarginalPriceFile.PRICE_SPAIN, 10.0],
        'Precio marginal en el sistema español (EUR/MWh)':
            [DataTypeInMarginalPriceFile.PRICE_SPAIN, 1.0],
        'Precio marginal en el sistema portugués (Cent/kWh)':
            [DataTypeInMarginalPriceFile.PRICE_PORTUGAL, 10.0],
        'Precio marginal en el sistema portugués (EUR/MWh)':
            [DataTypeInMarginalPriceFile.PRICE_PORTUGAL, 1.0],
        'Demanda+bombeos (MWh)':
            [DataTypeInMarginalPriceFile.ENERGY_IBERIAN, 1.0],
        'Energía en el programa resultante de la casación (MWh)':
            [DataTypeInMarginalPriceFile.ENERGY_IBERIAN, 1.0],
        'Energía total del mercado Ibérico (MWh)':
            [DataTypeInMarginalPriceFile.ENERGY_IBERIAN, 1.0],
        'Energía total con bilaterales del mercado Ibérico (MWh)':
            [DataTypeInMarginalPriceFile.ENERGY_IBERIAN_WITH_BILLATERAL, 1.0]}

    __key_list_retrieve__ = ['DATE', 'CONCEPT',
                             'H1', 'H2', 'H3', 'H4','H5', 'H6','H7', 'H8','H9','H10',
                             'H11', 'H12','H13', 'H14','H15', 'H16','H17', 'H18','H19','H20',
                             'H21', 'H22','H23', 'H24', "H25"]

    __dateFormatInFile__ = '%d/%m/%Y'
    __localeInFile__ = "en_DK.UTF-8"

    def __init__(self, types=None):
        self.conceptsToLoad = [v for v in DataTypeInMarginalPriceFile] if not types else types

    def get_keys(self):
        return MarginalPriceFileReader.__key_list_retrieve__

    def get_data_from_response(self, response: Response) -> pd.DataFrame:

        res = pd.DataFrame(columns=self.get_keys())

        # from first line we get the units and the price date. We just look at the date
        lines = response.text.split("\n")
        matches = re.findall('\d\d/\d\d/\d\d\d\d', lines.pop(0))
        if not (len(matches) == 2):
            print('Response ' + response.url + ' does not have the expected format.')
        else:
            # The second date is the one we want
            date = dt.datetime.strptime(matches[1], MarginalPriceFileReader.__dateFormatInFile__).date()

            # Process all the lines

            while lines:

                # read following line
                line = lines.pop(0)
                splits = line.split(sep=';')
                first_col = splits[0]

                if first_col in MarginalPriceFileReader.__dic_static_concepts__.keys():
                    concept_type = MarginalPriceFileReader.__dic_static_concepts__[first_col][0]

                    if concept_type in self.conceptsToLoad:
                        units = MarginalPriceFileReader.__dic_static_concepts__[first_col][1]

                        dico = self._process_line(date=date, concept=concept_type, values=splits[1:], multiplier=units)
                        res = pd.concat([res, pd.DataFrame([dico])], ignore_index=True)

            return res

    def get_data_from_file(self, filename: str) -> pd.DataFrame:

        # Method yield each dictionary one by one
        res = pd.DataFrame(columns=self.get_keys())
        file = open(filename, 'r', encoding='latin-1')

        # from first line we get the units and the price date. We just look at the date
        line = file.readline()
        matches = re.findall('\d\d/\d\d/\d\d\d\d', line)
        if not (len(matches) == 2):
            print('File ' + filename + ' does not have the expected format.')
        else:
            # The second date is the one we want
            date = dt.datetime.strptime(matches[1], MarginalPriceFileReader.__dateFormatInFile__).date()

            # Process all the lines
            while line:
                # read following line
                line = file.readline()
                splits = line.split(sep=';')
                first_col = splits[0]

                if first_col in MarginalPriceFileReader.__dic_static_concepts__.keys():
                    concept_type = MarginalPriceFileReader.__dic_static_concepts__[first_col][0]

                    if concept_type in self.conceptsToLoad:
                        units = MarginalPriceFileReader.__dic_static_concepts__[first_col][1]
                        dico = self._process_line(date=date, concept=concept_type, values=splits[1:], multiplier=units)
                        res = pd.concat([res, pd.DataFrame([dico])], ignore_index=True)

            return res

    def _process_line(self, date: dt.date, concept: DataTypeInMarginalPriceFile, values: list, multiplier=1.0) -> dict:

        key_list = MarginalPriceFileReader.__key_list_retrieve__

        result = dict.fromkeys(self.get_keys())
        result[key_list[0]] = date
        result[key_list[1]] = str(concept)

        for i, v in enumerate(values, start=1):

            if i > 25:
                break # Jump if 25-hour day or spaces ..
            try:
                f = multiplier * float(parse_decimal(v, locale=self.__localeInFile__))
            except:

                if i == 24:
                    # Day with 23-hours.
                    result[key_list[25]] = np.nan
                    result[key_list[26]] = np.nan
                elif i == 25:
                    # Day with 25-hours.
                    result[key_list[26]] = np.nan
                else:
                    raise
            else:
                result[key_list[i + 1]] = f

        return result



import pandas as pd
import datetime as dt

from OMIEData.DataImport.omie_data_importer import OMIEDataImporter
from OMIEData.FileReaders.omie_file_reader import OMIEFileReader
from OMIEData.Downloaders.omie_downloader import OMIEDownloader


class OMIEDataImporterFromResponses(OMIEDataImporter):

    def __init__(self,
                 date_ini: dt.date,
                 date_end: dt.date,
                 file_downloader: OMIEDownloader,
                 file_reader: OMIEFileReader):

        self.fileDownloader = file_downloader
        self.fileReader = file_reader
        self.date_ini = date_ini
        self.date_end = date_end

    def read_to_dataframe(self, verbose=False) -> pd.DataFrame:

        df = pd.DataFrame(columns=self.fileReader.get_keys())
        for response in self.fileDownloader.url_responses(date_ini=self.date_ini,
                                                          date_end=self.date_end,
                                                          verbose=verbose):
            try:
                df = pd.concat([df, self.fileReader.get_data_from_response(response=response)], ignore_index=True)

            except Exception as exc:
                print('There was error processing file: ' + response.url)
                print('{}'.format(exc) + response.url)
            else:
                if verbose:
                    print('Url: ' + response.url + ' successfully processed')

        return df



import datetime as dt

from OMIEData.DataImport.omie_data_importer_from_responses import OMIEDataImporterFromResponses
from OMIEData.Downloaders.marginal_price_downloader import MarginalPriceDownloader
from OMIEData.FileReaders.marginal_price_file_reader import MarginalPriceFileReader


class OMIEMarginalPriceFileImporter(OMIEDataImporterFromResponses):

    def __init__(self, date_ini: dt.date, date_end: dt.date):

        super().__init__(date_ini=date_ini,
                         date_end=date_end,
                         file_downloader=MarginalPriceDownloader(),
                         file_reader=MarginalPriceFileReader())



import pandas as pd
import os

from OMIEData.DataImport.omie_data_importer import OMIEDataImporter
from OMIEData.FileReaders.omie_file_reader import OMIEFileReader


class OMIEDataImporterFromFolder(OMIEDataImporter):

    def __init__(self, absolute_path: str, file_reader: OMIEFileReader):
        self.folder = absolute_path
        self.fileReader = file_reader

    def read_to_dataframe(self, verbose=False) -> pd.DataFrame:

        # List all the files in the directory
        filenames = [f for f in os.listdir(self.folder) if os.path.isfile(os.path.join(self.folder, f))]

        df = pd.DataFrame(columns=self.fileReader.get_keys())
        for f in filenames:
            try:
                df = pd.concat([df, self.fileReader.get_data_from_file(filename=os.path.join(self.folder, f))],
                               ignore_index=True)

            except Exception as exc:
                print('There was error processing file: ' + f)
                print('{}'.format(exc) + f)
            else:
                if verbose:
                    print('File: ' + f + ' successfully processed')

        return df



import pandas as pd
from requests import Response


class OMIEFileReader:

    @staticmethod
    def get_keys() -> list:
        pass

    def get_data_from_file(self, filename: str) -> pd.DataFrame:
        pass

    def get_data_from_response(self, response: Response) -> pd.DataFrame:
        pass




from OMIEData.Downloaders.general_omie_downloader import GeneralOMIEDownloader


class MarginalPriceDownloader(GeneralOMIEDownloader):

    url_year = 'AGNO_YYYY'
    url_month = '/MES_MM/TXT/'
    url_name = 'INT_PBC_EV_H_1_DD_MM_YYYY_DD_MM_YYYY.TXT'
    output_mask = 'PMD_YYYYMMDD.txt'

    def __init__(self):

        url1 = self.url_year + self.url_month + self.url_name
        GeneralOMIEDownloader.__init__(self, url_mask=url1, output_mask=self.output_mask)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="OMIEData",
    version="0.4.0.0",
    author="Alberto Cruz and Mirel Mora",
    author_email="a.cruz.g "
                 ""
                 ""
                 "arcia@gmail.com, mirel.mora@gmail.com",
    description="Package to download electricity time series from https://www.omie.es/",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/acruzgarcia/OMIEData",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires=">=3.10",
    keywords=["OMIE", "Electricity prices"],
    install_requires=["pandas>=2.0.1", "requests", "datetime", "babel"],
)



from OMIEData.Downloaders.general_omie_downloader import GeneralOMIEDownloader


class AdjustmentPriceDownloader(GeneralOMIEDownloader):
    url_year = "AGNO_YYYY"
    url_month = "/MES_MM/TXT/"
    url_name = "INT_MAJ_EV_H_DD_MM_YYYY_DD_MM_YYYY.TXT"
    output_mask = "PMD_YYYYMMDD.txt"

    def __init__(self):
        url1 = self.url_year + self.url_month + self.url_name
        GeneralOMIEDownloader.__init__(
            self, url_mask=url1, output_mask=self.output_mask
        )



import datetime as dt

from OMIEData.DataImport.omie_data_importer_from_responses import OMIEDataImporterFromResponses
from OMIEData.Downloaders.energy_by_technology_downloader import EnergyByTechnologyDownloader
from OMIEData.FileReaders.energy_by_technology_files_reader import EnergyByTechnologyHourlyFileReader
from OMIEData.Enums.all_enums import SystemType


class OMIEEnergyByTechnologyImporter(OMIEDataImporterFromResponses):

    def __init__(self, date_ini: dt.date, date_end: dt.date, system_type: SystemType):

        super().__init__(date_ini=date_ini,
                         date_end=date_end,
                         file_downloader=EnergyByTechnologyDownloader(system=system_type),
                         file_reader=EnergyByTechnologyHourlyFileReader())
